
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte
  module Agent
    module Configuration

      #
      # This agent reads a configuration file
      #
      #  class Agent < Tomte::Agent::Async
      #    read_configuration 'console.yml', :path => :system
      #    read_configuration 'console.yml', :path => :application
      #
      #  ...
      #
      def read_configuration(file, options = {})
        Tomte::Config.add_config_files(options.merge(:file => file))
      end

      #
      # Make the agent read an entire set of configuration files matching a glob
      #
      def read_configurations_matching(glob, options = {})
        Tomte::Config.add_config_files_matching(glob, options)
      end

      def has_system_worker(name, params)
        has_worker name, params, :system_workers
      end

      def has_system_workers(list)
        has_workers list, :system_workers
      end

      def has_worker(name, params, group = :workers)
        Tomte::Config.agents[group][name] = params
      end

      def has_workers(list, group = :workers)
         Tomte::Config.agents[group].merge list
      end

      def needs_privilege_management
        Tomte::Config.agents[:privilege_management] = true
      end

      def base_configuration(key, default = {}, &blk)
        Tomte.config.add key, default
        blk.call Tomte.config[key] if blk
      end

      def override_configuration(section, &blk)
        Tomte.config.override(section, &blk)
      end

      #
      #  class Agent < Tomte::Agent::Async
      #
      #    additional_options "My Agent"  do |config, opts|
      #      opts.on("--foo-bar PARAM", "Sets teh foobar to PARAM") do |param|
      #         config[:core][:foobar] = param
      #      end
      #    end
      #
      # ...
      #
      def additional_options(name, &blk)
        Tomte::Config.additional_options(name, &blk)
      end

      #
      #  class Agent < Tomte::Agent::Async
      #
      #    cli_commands :start, :stop
      #
      #  ...
      #
      def cli_commands(hash)
        Tomte::Config.additional_commands(hash)
      end

      #
      #  class Agent < Tomte::Agent::Async
      #
      #    help_text "This agent does this, and that"
      #
      #  ...
      #
      def help_text(text)
        Tomte::Config.additional_help_text(text)
      end

      #
      #
      def caption(text)
        Tomte::Config.core[:application_caption] = text
      end


    end
  end
end
