
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte::Agents

  module Configuration
    extend ActiveSupport::Concern

    included do
      prepend ConfigMethods
    end

    def log_path
     fsh_layout? ?
       File.join('/var', 'log', 'tomte') :
       File.join(core[:application_root], 'log')
    end

    def work_path
     fsh_layout? ?
       File.join('/var', 'run', 'tomte') :
       File.join(core[:application_root], 'tmp')
    end

    private
    #######

    #
    # true if the tomte application is running from a package, so we want to be fsh-compliant
    #
    def fsh_layout?
      !!core[:application_root].match(/(ruby|var).*gem/) || false
    end

    module ConfigMethods
      def post_configuration
        super
        if core[:interactive]
          agents[:daemonize] = false
          core[:log_profile] = :interactive
        end
        if agents[:daemonize]
          core[:log_path] ||= log_path
          core[:log_profile] = :daemonized
        end
        agents[:pid_path] ||= work_path
        agents[:pid_file] ||= File.join(agents[:pid_path],
          "%s-%s-%s.pid" % [core[:application], core[:environment], local[:identity]])

        agents[:start_directory] ||= Dir.getwd
      end
    end
  end
end

Tomte::Configuration.send(:include,Tomte::Agents::Configuration)