# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

require 'case'

module  Tomte
  class Actor

    #
    # Mailbox filterset.  Takes patterns or procs to match messages with
    # and returns the associated proc when a pattern matches.
    #
    class Filter

      #
      #
      #
      def initialize(params)
        @ruleset = []
      end

      #
      # Provide a pattern to match against with === and a block to call
      # when the pattern is matched.
      #
      def when(pattern, &action)
        # Don't explicitly require an action to be specified
        action ||= proc {}
        @ruleset.unshift [pattern, action]
      end

      #
      # Match a message using the filter
      #
      def match(message)
        _, action = @ruleset.find { |pattern, _| pattern === message }
        action
      end

      #
      # Is the filterset empty?
      #
      def empty?
        @ruleset.empty?
      end
    end
  end
end
