
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte
module Workers
  class Supervisor < Tomte::Worker

    #
    # called for every event coming for the bus
    #
    def event(e)
      # ignore events missing mandatory IEs
      return false if check_mandatory_ie(e)

      log.debug "+ supervisor: received an event type #{e.key}"
      trigger :event, e

      case e.key
        when 'event.presence.sign_on'
          event_agent_sign_on(e)

        when 'event.presence.hello'
          event_agent_hello(e)

        when 'event.presence.sign_off'
          event_agent_bye(e)

        when 'event.presence.workers_update'
          event_agent_workers_update(e)

        else
          log.debug '+ supervisor:  unknown event'
      end

    end

    def event_agent_sign_on(e)


      if registry.includes(e.header.uuid)

        #log.warn "+ duplicate signon from #{e.header.director}"
        # what should we do here? refuse it? compare it? accept and take over?

        # TODO silently ignore, so we can propagate signons on the event bus and keep all
        #      the supervisors in sync

      else
        log.info "+ registry, agent #{e.header.director} is signing on"
        agent_learn(e.header.uuid, e.header)
      end
    end


    def event_agent_sign_off(e)

      agent_forget(e.header.uuid)
    end

    def event_agent_hello(e)

      if registry.includes(e.header.uuid)
        agent_touch(e.header.uuid)
      else
        #hello from an agent we don't know about

      end
    end



  end
end
end
