# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte::Network
  module API

    autoload  :BasicServer,  'tomte/network/api/basic_server'
    autoload  :Server,       'tomte/network/api/server'

    extend ActiveSupport::Concern

    included do
      class_attribute :declared_apis
      self.declared_apis ||= {}

      class << self
        prepend Inheritance
      end
    end

    protected
    #########

    module Inheritance
      def inherited(subclass)
        super(subclass)
        subclass.declared_apis = self.declared_apis.dup
      end
    end

    module ClassMethods

      #
      # declares an api in this worker class
      #
      def api(name, options = {}, &block)

        buses[options[:bus]] ||= {:options => {}}
        declared_apis[name] = {:options => options, :block => block}

      end
      alias :has_api :api
    end


    #
    # returns the api with :name from this instance of worker
    #
    def api(name)
      apis[name]
    end

    private
    #######

    def apis
      @apis ||= {}
    end

    def initialize_apis

      append_callback :all_endpoints_ready, &method(:start_apis)

      declared_apis.each do |name, declared_api|
        new_rpc_api name, declared_api[:options], &declared_api[:block]
      end
    end

    def new_rpc_api(name, options = {}, &block)
      apis[name] = Instance.new(name, self, options, &block)
      apis[name]
    end

    def start_apis
      log.debug "+ Starting all the apis for this worker."
      apis.each {|_, a| a.start! }
    end

    class Instance

      attr_reader :options
      attr_reader :server
      attr_reader :rpc_server

      def initialize(name, worker, options = {}, &block)

        raise ArgumentError, "API with no Server?" if(options[:class].blank? && options[:server].blank?)

        @options = options.dup
        @bus = @options.delete(:bus)
        @name = name
        @worker = worker
        @server = @options[:server] ||= Kernel.constant(options[:class]).new(worker, options.merge(:name => name))
        @callback = block
      end

      def start!
        @endpoint = @worker.endpoint(@bus)
        raise ArgumentError, "Starting an API that is not connected to an endpoint." unless @endpoint

        @rpc_server = @endpoint.rpc_server(@options).ready! do |rpc|
          instance_exec(rpc, &@callback) if @callback
        end
      end

      def stop!(&block)
        @rpc_server.close(&callback)
      end


    end
  end
end
