
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

require 'tomte/async/supervisor/agent_registry'
require 'tomte/async/supervisor/api'
require 'tomte/async/supervisor/agent_interface'

require 'tomte/async/supervisor/events'


module Tomte
module Workers
  class Supervisor < Tomte::Worker

    attr_accessor :registry

    worker :system => true,
           :bus => :management


    protected
    #########


    def init(options = {})

      @heartbeat = config.core[:heartbeat] || 30
      @registry = Tomte::Workers::Supervisor::AgentRegistry.new(log, @heartbeat)
      @registry.reset! if config[:supervisor][:reset_registry]

      @agent_interface_queue = config[:supervisor][:agent_interface] || 'tomte.supervisor.agent'
      @api_interface_queue = config[:supervisor][:api_interface] || 'tomte.supervisor.api'

      @callbacks={}

      start_heartbeat_timer

#      start_event_listener

      # publish the rpc interface for agents
      has_api :agent_interface, :queue => @agent_interface_queue,
        :server => Tomte::Workers::Supervisor::AgentInterface.new(self, registry),
        :raw => true

      # publish the async api
      has_api :supervisor_api, :queue => @api_interface_queue,
        :server => Tomte::Workers::Supervisor::API.new(self, registry)

      #publish_agent_interface
      # publish the async api
      #publish_api

      trigger :ready
    end


    public
    ######

    #
    # registers a callback for event
    #
    def on_event(signal, &blk)
      @callbacks[signal] = blk
    end


    private
    #######

    # def publish_api
    #   log.debug "+ Supervisor is listening for API RPC on '#{@api_interface_queue}'"
    #   @api = Tomte::Workers::Supervisor::API.new(self, registry)
    #   bus.xrpc(:queue => @api_interface_queue, :server => @api)
    # end
    #
    # def publish_agent_interface
    #   log.debug "+ Supervisor is listening for agent RPC on '#{@agent_interface_queue}'"
    #   @agent_interface = Tomte::Workers::Supervisor::AgentInterface.new(self, registry)
    #   bus.xrpc(:queue => @agent_interface_queue, :server => @agent_interface, :raw => true, :log => log)
    # end

    #
    # starts the periodic timer
    #
    def start_heartbeat_timer
      @hb_timer = EventMachine::PeriodicTimer.new(@heartbeat) do
        heartbeat
      end
    end

    #
    # hb timer
    #
    def heartbeat

      registry.tick(Time.now.to_i)
      log.debug '+ heartbeat'
      log.debug "\n" + registry.to_human
    end


    def trigger(signal, *params)
      @callbacks[signal].call(*params) unless @callbacks[signal].nil?
    end

  end
end
end
